@@ -6,6 +6,7 @@
 #include "minefield.h"
 #include "flag.xpm"
 #include "mine.xpm"
+#include "question.xpm"
 
 static struct {
 	gint x;
@@ -23,16 +24,16 @@ static struct {
 
 
 
-static int num_colors[9][3] = {
-	{ 0  , 0  , 0   }, /* Black, not used */
-	{ 0  , 0  , 255 }, /* Blue  */
-	{ 0  , 160, 0   }, /* Green */
-	{ 255, 0  , 0   }, /* Red   */
-	{ 0  , 0  , 127 }, /* DarkBlue */
-	{ 160, 0  , 0   }, /* DarkRed   */
-	{ 0  , 255, 255 }, /* Cyan */
-	{ 160, 0  , 160 }, /* DarkViolet */
-	{ 0  , 0  , 0   }  /* Black */
+static guint16 num_colors[9][3] = {
+	{ 0x0000, 0x0000, 0x0000 }, /* Black, not used */
+	{ 0x0000, 0x0000, 0xffff }, /* Blue  */
+	{ 0x0000, 0xa0a0, 0x0000 }, /* Green */
+	{ 0xffff, 0x0000, 0x0000 }, /* Red   */
+	{ 0x0000, 0x0000, 0x7fff }, /* DarkBlue */
+	{ 0xa0a0, 0x0000, 0x0000 }, /* DarkRed   */
+	{ 0x0000, 0xffff, 0xffff }, /* Cyan */
+	{ 0xa0a0, 0x0000, 0xa0a0 }, /* DarkViolet */
+	{ 0x0000, 0x0000, 0x0000 }  /* Black */
 };
 
 
@@ -48,9 +49,38 @@ enum {
 };
 
 static gint minefield_signals[LAST_SIGNAL] = { 0 };
-
 static GtkWidgetClass *parent_class;
 
+/*  Prototypes */
+static gulong get_random(gulong limit);
+static inline gint cell_idx(GtkMineField *mfield, guint x, guint y);
+static void gtk_mine_draw(GtkMineField *mfield, guint x, guint y);
+static gint gtk_minefield_button_press(GtkWidget *widget, GdkEventButton *event);
+static gint gtk_minefield_button_release(GtkWidget *widget, GdkEventButton *event);
+static void gtk_minefield_check_field(GtkMineField *mfield, gint x, gint y);
+static void gtk_minefield_class_init (GtkMineFieldClass *class);
+static gint gtk_minefield_expose(GtkWidget *widget, GdkEventExpose *event);
+static void gtk_minefield_init (GtkMineField *mfield);
+static void gtk_minefield_lose(GtkMineField *mfield);
+static gint gtk_minefield_motion_notify(GtkWidget *widget, GdkEventMotion *event);
+static void gtk_minefield_multi_release (GtkMineField *mfield, guint x, guint y, guint c, guint really);
+static void gtk_minefield_randomize(GtkMineField *mfield, int curloc);
+static void gtk_minefield_realize(GtkWidget *widget);
+static void gtk_minefield_setup_signs(GtkWidget *widget);
+static void gtk_minefield_show(GtkMineField *mfield, guint x, guint y);
+static void gtk_minefield_size_allocate(GtkWidget *widget, GtkAllocation *allocation);
+static void gtk_minefield_size_request(GtkWidget *widget, GtkRequisition *requisition);
+static void gtk_minefield_toggle_mark(GtkMineField *mfield, guint x, guint y);
+static void gtk_minefield_unrealize (GtkWidget *widget);
+static void gtk_minefield_win(GtkMineField *mfield);
+static void init_random(gulong seed);
+static inline int gtk_minefield_check_cell(GtkMineField *mfield, guint x, guint y);
+static void _setup_sign (sign *signp, const char **data, guint minesize);
+static inline void gtk_minefield_multi_press(GtkMineField *mfield, guint x, guint y, gint c);
+/* end prototypes */
+
+
+
 static inline gint cell_idx(GtkMineField *mfield, guint x, guint y)
 {
 	if (x>=0 && x<mfield->xsize && y>=0 && y<mfield->ysize)
@@ -68,7 +98,7 @@ static void _setup_sign (sign *signp, const char **data, guint minesize)
 					   &signp->mask, 127);
 
 	gdk_pixbuf_unref (image);
-        gdk_window_get_size(signp->pixmap, &(signp->width), &(signp->height));
+        gdk_drawable_get_size(signp->pixmap, &(signp->width), &(signp->height));
 }
 
 static void gtk_minefield_setup_signs(GtkWidget *widget)
@@ -79,6 +109,72 @@ static void gtk_minefield_setup_signs(GtkWidget *widget)
   
         _setup_sign(&mfield->flag, flag_xpm, mfield->minesize);
         _setup_sign(&mfield->mine, mine_xpm, mfield->minesize);
+	_setup_sign(&mfield->question, question_xpm, mfield->minesize);
+}
+
+static void
+gtk_minefield_setup_numbers(GtkMineField *mfield)
+{
+	int minesize, pixel_sz, i;
+	  
+	minesize = mfield->minesize;
+
+	pixel_sz = minesize - 2;
+	if (pixel_sz > 999) pixel_sz = 999;
+	if (pixel_sz < 2)  pixel_sz = 2;
+  
+	for (i=0; i<9; i++) {
+		gchar text[2];
+		PangoLayout *layout;
+		PangoAttrList *alist;
+		PangoAttribute *attr;
+		PangoFontDescription *font_desc;
+
+		/* free an existing layout ... */
+		if (mfield->numstr[i].layout)
+			g_object_unref(mfield->numstr[i].layout);
+
+		text[0] = '0' + i;
+		text[1] = '\0';
+		layout = gtk_widget_create_pango_layout(GTK_WIDGET(mfield),
+							text);
+
+		/* set attributes for the layout */
+		alist = pango_attr_list_new();
+
+		/* do the font */
+		font_desc = pango_font_description_new();
+		pango_font_description_set_family(font_desc, "Courier 8");
+		pango_font_description_set_size(font_desc,
+						pixel_sz * PANGO_SCALE);
+		pango_font_description_set_weight(font_desc,PANGO_WEIGHT_BOLD);
+		attr = pango_attr_font_desc_new(font_desc);
+		pango_font_description_free(font_desc);
+
+		attr->start_index = 0;
+		attr->end_index = G_MAXUINT;
+		pango_attr_list_insert(alist, attr);
+
+		/* colour */
+		attr = pango_attr_foreground_new(num_colors[i][0],
+						 num_colors[i][1],
+						 num_colors[i][2]);
+		attr->start_index = 0;
+		attr->end_index = G_MAXUINT;
+		pango_attr_list_insert(alist, attr);
+
+		pango_layout_set_attributes(layout, alist);
+		pango_attr_list_unref(alist);
+
+		pango_layout_set_width(layout, minesize);
+		pango_layout_set_alignment(layout, PANGO_ALIGN_CENTER);
+
+		mfield->numstr[i].layout = layout;
+
+		mfield->numstr[i].dx = minesize / 2;
+		/* Hack to fix the number being too far down */
+		mfield->numstr[i].dy = (minesize - pixel_sz) / 2 - 4;
+	}
 }
 
 static void gtk_minefield_realize(GtkWidget *widget)
@@ -102,13 +198,15 @@ static void gtk_minefield_realize(GtkWidget *widget)
         attributes.visual = gtk_widget_get_visual(widget);
         attributes.colormap = gtk_widget_get_colormap(widget);
         attributes.event_mask = gtk_widget_get_events(widget);
-	attributes.event_mask |= GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK |
+	attributes.event_mask |= GDK_EXPOSURE_MASK |
+	                         GDK_BUTTON_PRESS_MASK |
 		                 GDK_BUTTON_RELEASE_MASK |
                                  GDK_POINTER_MOTION_MASK;
         
         attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
         
-        widget->window = gdk_window_new(widget->parent->window, &attributes, attributes_mask);
+	widget->window = gdk_window_new(widget->parent->window,
+	                                &attributes, attributes_mask);
         gdk_window_set_user_data(widget->window, mfield);
         
         widget->style = gtk_style_attach(widget->style, widget->window);
@@ -154,6 +252,7 @@ static void gtk_minefield_size_request(GtkWidget *widget,
 {
         requisition->width  = GTK_MINEFIELD(widget)->xsize *
 		              GTK_MINEFIELD(widget)->minesize;
+
 	requisition->height = GTK_MINEFIELD(widget)->ysize * 
 		              GTK_MINEFIELD(widget)->minesize;
 }
@@ -175,53 +274,82 @@ static void gtk_mine_draw(GtkMineField *mfield, guint x, guint y)
 		noshadow = mfield->mines[c].down || mfield->mines[c].shown;
 	}
 	
-	gdk_window_clear_area(widget->window,
-			      x*minesize, y*minesize,
-			      minesize,
-			      minesize);
+	gdk_window_clear_area(widget->window, x * minesize, y * minesize,
+	                      minesize, minesize);
 
-        if (!noshadow)
-        {
-                gtk_draw_shadow(widget->style, widget->window,
-                                GTK_WIDGET_STATE (widget), GTK_SHADOW_OUT,
-                                x*minesize, y*minesize,
-                                minesize,
-                                minesize);
-        } else {
-                gdk_draw_line(widget->window,
-                              widget->style->black_gc,
+	if (noshadow) { /* draw grid on ocean floor */
+		if (y == 0) {	/* top row only */
+			gdk_draw_line(widget->window,	/* top */
+			              widget->style->dark_gc[2],
+	                              x*minesize,
+	                              0,
+	                              x*minesize+minesize-1,
+	                              0);
+		}
+		if (x == 0) {	/* left column only */
+			gdk_draw_line(widget->window,	/* left */
+			              widget->style->dark_gc[2],
+	                              0,
+	                              y*minesize,
+	                              0,
+	                              y*minesize+minesize-1);
+		}
+		gdk_draw_line(widget->window,	/* right */
+		              widget->style->dark_gc[2],
                               x*minesize+minesize-1,
                               y*minesize,
                               x*minesize+minesize-1,
                               y*minesize+minesize-1);
-                gdk_draw_line(widget->window,
-                              widget->style->black_gc,
+		gdk_draw_line(widget->window,	/* bottom */
+		              widget->style->dark_gc[2],
                               x*minesize,
                               y*minesize+minesize-1,
                               x*minesize+minesize-1,
                               y*minesize+minesize-1);
+
+	} else {	/* draw shadow around possible mine location */
+		gtk_paint_shadow(widget->style, widget->window,
+		                GTK_WIDGET_STATE (widget), GTK_SHADOW_OUT,
+				NULL, widget, NULL,
+		                x*minesize, y*minesize, minesize, minesize);
         }
 
 	if (mfield->mines[c].shown && !mfield->mines[c].mined) {
 		if ((n = mfield->mines[c].neighbours) != 0) {
-			gdk_draw_string(widget->window,
-					mfield->font,
-					mfield->numstr[n].gc,
-					x*minesize+mfield->numstr[n].dx,
-					y*minesize+mfield->numstr[n].dy,
-					mfield->numstr[n].text);
+			g_assert (n >= 0 && n <= 9);
+			gdk_draw_layout(widget->window,
+					widget->style->black_gc,
+					x*minesize + mfield->numstr[n].dx,
+					y*minesize + mfield->numstr[n].dy,
+					PANGO_LAYOUT(mfield->numstr[n].layout));
 		}
-	} else if (mfield->mines[c].marked == 1) {
+
+	} else if (mfield->mines[c].marked == MINE_QUESTION) {
+		gdk_gc_set_clip_mask(widget->style->black_gc,
+		                     mfield->question.mask);
+		gdk_gc_set_clip_origin(widget->style->black_gc,
+		                       x * minesize + (minesize - mfield->question.width) / 2, 
+		                       y * minesize + (minesize - mfield->question.height) / 2);
+		gdk_draw_drawable (widget->window,
+		                 widget->style->black_gc,
+		                 mfield->question.pixmap, 0, 0,
+		                 x * minesize + (minesize - mfield->question.width) / 2, 
+		                 y * minesize + (minesize - mfield->question.height) / 2,
+		                 -1, -1);
+		gdk_gc_set_clip_mask(widget->style->black_gc, NULL);
+	  
+	} else if (mfield->mines[c].marked == MINE_MARKED) {
 		gdk_gc_set_clip_mask(widget->style->black_gc,
 					     mfield->flag.mask);
 		gdk_gc_set_clip_origin(widget->style->black_gc,
 					       x * minesize + (minesize - mfield->flag.width) / 2, 
 					       y * minesize + (minesize - mfield->flag.height) / 2);
-		gdk_draw_pixmap (widget->window,
+		gdk_draw_drawable (widget->window,
 				 widget->style->black_gc,
-				 mfield->flag.pixmap,
-				 0, 0, x * minesize + (minesize - mfield->flag.width) / 2, 
-				       y * minesize + (minesize - mfield->flag.height) / 2, -1, -1);
+		                 mfield->flag.pixmap, 0, 0,
+		                 x * minesize + (minesize - mfield->flag.width) / 2, 
+		                 y * minesize + (minesize - mfield->flag.height) / 2,
+		                 -1, -1);
 	        gdk_gc_set_clip_mask(widget->style->black_gc, NULL);
 	  
 		if (mfield->lose && mfield->mines[c].mined != 1) {
@@ -250,7 +378,8 @@ static void gtk_mine_draw(GtkMineField *mfield, guint x, guint y)
 				      x*minesize+minesize-3,
 				      y*minesize+3);
 		}
-	} else if ( mfield->lose && mfield->mines[c].mined) {
+
+	} else if (mfield->lose && mfield->mines[c].mined) {
 		if (mfield->mine.mask) {
 			gdk_gc_set_clip_mask(widget->style->black_gc,
 					     mfield->mine.mask);
@@ -259,11 +388,12 @@ static void gtk_mine_draw(GtkMineField *mfield, guint x, guint y)
 					       y*minesize + (minesize - mfield->mine.height) / 2);
 		}
 
-	        gdk_draw_pixmap (widget->window,
+	        gdk_draw_drawable (widget->window,
 				 widget->style->black_gc,
-				 mfield->mine.pixmap,
-				 0, 0, x * minesize + (minesize - mfield->mine.width) / 2,
-				       y * minesize + (minesize - mfield->mine.height) / 2, -1, -1);
+		                 mfield->mine.pixmap, 0, 0,
+		                 x * minesize + (minesize - mfield->mine.width) / 2,
+		                 y * minesize + (minesize - mfield->mine.height) / 2,
+		                 -1, -1);
 		
 		if (mfield->flag.mask) {
 			gdk_gc_set_clip_mask(widget->style->black_gc, NULL);
@@ -297,10 +427,7 @@ static gint gtk_minefield_expose(GtkWidget *widget,
 				 GdkEventExpose *event)
 {
         GtkMineField *mfield;
-	GdkColor color;
-	gint n;
 	guint minesize;
-        int i;
 
 	g_return_val_if_fail(widget != NULL, FALSE);
         g_return_val_if_fail(GTK_IS_MINEFIELD(widget), FALSE);
@@ -313,45 +440,6 @@ static gint gtk_minefield_expose(GtkWidget *widget,
 	
 	minesize = mfield->minesize;
 
-	if (mfield->numstr[0].gc == 0) {
-	        int pxlsz;
-	        char fontname[50];
-	  
-	        gtk_minefield_setup_signs(widget);
-
-	        pxlsz = minesize - 2;
-	        if (pxlsz > 999) pxlsz = 999;
-                if (pxlsz < 2)  pxlsz = 2;
-  
-                sprintf(fontname, "-bitstream-courier-bold-r-*-*-%d-*-*-*-*-*-*-*", pxlsz);
-	
-		mfield->font = gdk_font_load(fontname);
-	  
-	            /* The font used to be "-misc-fixed-bold-r-normal--13-*-*-*-*-*-*" */
-#if 0	  
-                if (!mfield->font) mfield->font = widget->style->font;
-#endif
-		for (i=0; i<9; i++) {
-			mfield->numstr[i].text[0] = i+'0';
-			mfield->numstr[i].text[1] = '\0';
-			mfield->numstr[i].dx =
-				(minesize-gdk_string_width(mfield->font,
-							   mfield->numstr[i].text))/2;
-			mfield->numstr[i].dy = (minesize + 5 * pxlsz / 8) / 2;
-			mfield->numstr[i].gc = gdk_gc_new(GTK_WIDGET(mfield)->window);
-
-			color.red   = num_colors[i][0] | (num_colors[i][0] << 8);
-			color.green = num_colors[i][1] | (num_colors[i][1] << 8);
-			color.blue  = num_colors[i][2] | (num_colors[i][2] << 8);
-			color.pixel = 0; /* required! */
-
-			n = 0;
-			
-			gdk_rgb_find_color (gtk_widget_get_colormap (widget), &color);
-			
-			gdk_gc_set_foreground(mfield->numstr[i].gc, &color);
-		}
-	}
 
 	gtk_minefield_draw(GTK_MINEFIELD(widget), &event->area);
 
@@ -372,7 +460,7 @@ static inline int gtk_minefield_check_cell(GtkMineField *mfield, guint x, guint
                 ny = y+neighbour_map[i].y;
                 if ((c = cell_idx(mfield, nx, ny)) != -1) {
                         if (mfield->mines[c].shown == 0 &&
-                            mfield->mines[c].marked == 0) {
+			    mfield->mines[c].marked == MINE_NOMARK) {
 				mfield->mines[c].shown = 1;
                                 mfield->shown++;
 				gtk_mine_draw(mfield, nx, ny);
@@ -427,10 +515,11 @@ static void gtk_minefield_check_field(GtkMineField *mfield, gint x, gint y)
         } while (changed);
 }
 
-static void gtk_minefield_loose(GtkMineField *mfield)
+static void gtk_minefield_lose(GtkMineField *mfield)
 {
-        gtk_signal_emit(GTK_OBJECT(mfield),
-                        minefield_signals[EXPLODE_SIGNAL]);
+        g_signal_emit(GTK_OBJECT(mfield),
+                        minefield_signals[EXPLODE_SIGNAL],
+			0, NULL );
         mfield->lose = 1;
         gtk_minefield_draw(mfield, NULL);
 }
@@ -439,32 +528,88 @@ static void gtk_minefield_win(GtkMineField *mfield)
 {
         guint x, y, c;
 
-	for (x = 0; x < mfield->xsize; x++)
-		for (y = 0; y < mfield->ysize; y++)
-                {
-                        c = x+y*mfield->xsize;
-                        if (mfield->mines[c].shown == 0 && mfield->mines[c].marked == 0)
-                        {
-                                mfield->mines[c].marked = 1;
-                                gtk_mine_draw(mfield, x, y);
+	/* mark any unmarked mines and update displayed total */
+	for (x = 0; x < mfield->xsize; x++) {
+		for (y = 0; y < mfield->ysize; y++) {
+			c = x + y * mfield->xsize;
+			if (mfield->mines[c].shown == 0 && /* not shown & not marked */
+			    mfield->mines[c].marked != MINE_MARKED) {
+
+				mfield->mines[c].marked = MINE_MARKED; /* mark it */
+				gtk_mine_draw(mfield, x, y);           /* draw it */
+				mfield->flag_count++;                  /* up the count */
+				g_signal_emit(GTK_OBJECT(mfield),    /* display the count */
+						minefield_signals[MARKS_CHANGED_SIGNAL],
+						0, NULL);
+			}
                         }
                 }
 
-        gtk_signal_emit(GTK_OBJECT(mfield),
-                        minefield_signals[WIN_SIGNAL]);
+	/* now stop the clock.  (MARKS_CHANGED_SIGNAL starts it) */
+        g_signal_emit(GTK_OBJECT(mfield),
+                        minefield_signals[WIN_SIGNAL],
+			0, NULL);
+
         mfield->win = 1;
 }
 
+static void gtk_minefield_randomize(GtkMineField *mfield, int curloc)
+{
+	guint i, j;
+	guint x, y;
+	guint tmp;
+	guint n;
+	guint cidx;
+
+	/* make sure curloc is not mined */
+	mfield->mines[mfield->xsize * mfield->ysize -1].mined =
+		mfield->mines[curloc].mined; /* last location is never mined */
+	mfield->mines[curloc].mined = 0;
+
+	/* swap mines randomly */
+	for (i=0; i<mfield->xsize * mfield->ysize; i++) {
+		j = (guint)get_random(mfield->xsize * mfield->ysize);
+		if (i == curloc || j == curloc) { /* skip current location */
+			continue;
+		}
+		tmp = mfield->mines[i].mined;
+		mfield->mines[i].mined = mfield->mines[j].mined;
+		mfield->mines[j].mined = tmp;
+	}
+
+	/* load neighborhood numbers */
+	for (x=0; x<mfield->xsize; x++) {
+		for (y=0; y<mfield->ysize; y++) {
+			n = 0;
+			for (i=0; i<8; i++) {
+				if (((cidx = cell_idx(mfield, x + neighbour_map[i].x,
+				                      y+neighbour_map[i].y)) != -1) &&
+					mfield->mines[cidx].mined) {
+					n++;
+				}
+			}
+			mfield->mines[x+mfield->xsize * y].neighbours = n;
+		}
+	}
+}
+
 static void gtk_minefield_show(GtkMineField *mfield, guint x, guint y)
 {
-	int c = x+mfield->xsize*y;
+	int c = x + mfield->xsize * y;
+
+	/* make sure first click isn't on a mine */
+	if (!mfield->in_play) {
+		mfield->in_play = 1;
+		gtk_minefield_randomize(mfield, c);
+	}
 	
-        if (mfield->mines[c].marked != 1 && mfield->mines[c].shown != 1) {
+	if (mfield->mines[c].marked != MINE_MARKED &&
+	    mfield->mines[c].shown != 1) {
 		mfield->mines[c].shown = 1;
 		mfield->shown++;
                 gtk_mine_draw(mfield, mfield->cdownx, mfield->cdowny);
                 if(mfield->mines[c].mined == 1) {
-                        gtk_minefield_loose(mfield);
+			gtk_minefield_lose(mfield);
                 } else {
 			gtk_minefield_check_field(mfield, x, y);
 			if (mfield->shown == mfield->xsize*mfield->ysize-mfield->mcount) {
@@ -477,21 +622,38 @@ static void gtk_minefield_show(GtkMineField *mfield, guint x, guint y)
 static void gtk_minefield_toggle_mark(GtkMineField *mfield, guint x, guint y)
 {
         int c = cell_idx(mfield, x, y);
-        if (mfield->mines[c].shown == 0) {
-		if ((mfield->mines[c].marked = 1-mfield->mines[c].marked) == 1) {
-			mfield->flags++;
-		} else {
-			mfield->flags--;
+
+	if (mfield->mines[c].shown != 0) { /* nothing to toggle */
+		return;
+	}
+
+	switch (mfield->mines[c].marked) {
+	case MINE_NOMARK:
+		mfield->mines[c].marked = MINE_MARKED;
+		mfield->flag_count++;
+		break;
+	case MINE_MARKED:
+		mfield->mines[c].marked = MINE_QUESTION;
+		mfield->flag_count--;
+		break;
+	case MINE_QUESTION:
+		mfield->mines[c].marked = MINE_NOMARK;
+		break;
+	default:
+		/* better not get here! */
+		break;
 		}
-		gtk_signal_emit(GTK_OBJECT(mfield),
-				minefield_signals[MARKS_CHANGED_SIGNAL]);
-                if (mfield->shown == mfield->xsize*mfield->ysize-mfield->mcount) {
+
+		g_signal_emit(GTK_OBJECT(mfield),
+				minefield_signals[MARKS_CHANGED_SIGNAL],
+				0, NULL);
+	if (mfield->shown == mfield->xsize * mfield->ysize-mfield->mcount) {
                         gtk_minefield_win(mfield);
 		}
-        }
 }
 
-static inline void gtk_minefield_multi_press(GtkMineField *mfield, guint x, guint y, gint c)
+static inline void gtk_minefield_multi_press(GtkMineField *mfield,
+                                             guint x, guint y, gint c)
 {
         guint i;
         gint nx, ny, c2;
@@ -501,7 +663,8 @@ static inline void gtk_minefield_multi_press(GtkMineField *mfield, guint x, guin
                 ny = y+neighbour_map[i].y;
                 if ((c2 = cell_idx(mfield, nx, ny)) == -1)
                         continue;
-                if (!mfield->mines[c2].marked && !mfield->mines[c2].shown) {
+		if (mfield->mines[c2].marked != MINE_MARKED &&
+		    !mfield->mines[c2].shown) {
                         mfield->mines[c2].down = 1;
                         gtk_mine_draw(mfield, nx, ny);
                 }
@@ -512,7 +675,7 @@ static inline void gtk_minefield_multi_press(GtkMineField *mfield, guint x, guin
 static void gtk_minefield_multi_release (GtkMineField *mfield, guint x, guint y, guint c, guint really)
 {
         gint n, nx, ny, i, c2;
-        guint loose = 0;
+	guint lose = 0;
 
         mfield->multi_mode = 0;
 
@@ -522,11 +685,13 @@ static void gtk_minefield_multi_release (GtkMineField *mfield, guint x, guint y,
                 ny = y+neighbour_map[i].y;
                 if ((c2 = cell_idx(mfield, nx, ny)) == -1)
                         continue;
-                if (mfield->mines[c2].marked) n++;
+		if (mfield->mines[c2].marked == MINE_MARKED)
+			n++;
         }
         if (mfield->mines[c].neighbours != n ||
-            mfield->mines[c].marked ||
-            !mfield->mines[c].shown) really = 0;
+	    mfield->mines[c].marked == MINE_MARKED ||
+	    !mfield->mines[c].shown)
+			really = 0;
 
         for (i=0; i<8; i++) {
                 nx = x+neighbour_map[i].x;
@@ -539,14 +704,14 @@ static void gtk_minefield_multi_release (GtkMineField *mfield, guint x, guint y,
 				mfield->mines[c2].shown = 1;
 				mfield->shown++;
                                 if (mfield->mines[c2].mined == 1) {
-                                        loose = 1;
+					lose = 1;
                                 }
                         }
                         gtk_mine_draw(mfield, nx ,ny);
                 }
         }
-        if (loose) {
-                gtk_minefield_loose(mfield);
+	if (lose) {
+		gtk_minefield_lose(mfield);
         } else if (really) {
                 gtk_minefield_check_field(mfield, x, y);
                 if (mfield->shown == mfield->xsize*mfield->ysize-mfield->mcount) {
@@ -587,14 +752,18 @@ static gint gtk_minefield_motion_notify(GtkWidget *widget, GdkEventMotion *event
                         gtk_mine_draw(mfield, mfield->cdownx, mfield->cdowny);
 
                         multi = mfield->multi_mode;
-                        if (multi) gtk_minefield_multi_release(mfield, mfield->cdownx, mfield->cdowny, mfield->cdown, 0);
+			if (multi)
+				gtk_minefield_multi_release(mfield, mfield->cdownx,
+				                            mfield->cdowny,
+				                            mfield->cdown, 0);
                         mfield->cdownx = x;
                         mfield->cdowny = y;
                         mfield->cdown = c;
                         mfield->mines[c].down = 1;
                         gtk_mine_draw(mfield, x, y);
 
-                        if (multi) gtk_minefield_multi_press(mfield, x, y, c);
+			if (multi)
+				gtk_minefield_multi_press(mfield, x, y, c);
                 }
         }
         return FALSE;
@@ -639,8 +808,9 @@ static gint gtk_minefield_button_press(GtkWidget *widget, GdkEventButton *event)
                         gtk_mine_draw(mfield, x, y);
                 }
 		if (event->button == 1 || event->button == 2) {
-			gtk_signal_emit(GTK_OBJECT(mfield),
-					minefield_signals[LOOK_SIGNAL]);
+			g_signal_emit(GTK_OBJECT(mfield),
+					minefield_signals[LOOK_SIGNAL],
+					0, NULL);
 		}
         }
         return FALSE;
@@ -675,8 +845,9 @@ static gint gtk_minefield_button_release(GtkWidget *widget, GdkEventButton *even
                         break;
                 }
 		if (!mfield->lose && !mfield->win) {
-			gtk_signal_emit(GTK_OBJECT(mfield),
-					minefield_signals[UNLOOK_SIGNAL]);
+			g_signal_emit(GTK_OBJECT(mfield),
+					minefield_signals[UNLOOK_SIGNAL],
+					0, NULL);
 		}
 		mfield->mines[mfield->cdown].down = 0;
                 mfield->cdown = -1;
@@ -713,62 +884,79 @@ static void gtk_minefield_class_init (GtkMineFieldClass *class)
         class->win = NULL;
   
 	minefield_signals[MARKS_CHANGED_SIGNAL] =
-		gtk_signal_new("marks_changed",
-			       GTK_RUN_FIRST,
-			       GTK_CLASS_TYPE (object_class),
-			       GTK_SIGNAL_OFFSET(GtkMineFieldClass, marks_changed),
-			       gtk_signal_default_marshaller,
-			       GTK_TYPE_NONE,
+		g_signal_new("marks_changed",
+				G_OBJECT_CLASS_TYPE (object_class),
+			       G_SIGNAL_RUN_FIRST,
+			       G_STRUCT_OFFSET(GtkMineFieldClass, marks_changed),
+			       NULL, NULL,
+			       g_cclosure_marshal_VOID__VOID,
+			       G_TYPE_NONE,
 			       0);
+
 	minefield_signals[EXPLODE_SIGNAL] =
-		gtk_signal_new("explode",
-			       GTK_RUN_FIRST,
-			       GTK_CLASS_TYPE (object_class),
-			       GTK_SIGNAL_OFFSET(GtkMineFieldClass, explode),
-			       gtk_signal_default_marshaller,
-			       GTK_TYPE_NONE,
+		g_signal_new("explode",
+				G_OBJECT_CLASS_TYPE (object_class),
+			       G_SIGNAL_RUN_FIRST,
+			       G_STRUCT_OFFSET(GtkMineFieldClass, explode),
+			       NULL, NULL,
+			       g_cclosure_marshal_VOID__VOID,
+			       G_TYPE_NONE,
 			       0);
 	minefield_signals[LOOK_SIGNAL] =
-		gtk_signal_new("look",
-			       GTK_RUN_FIRST,
-			       GTK_CLASS_TYPE (object_class),
-			       GTK_SIGNAL_OFFSET(GtkMineFieldClass, look),
-			       gtk_signal_default_marshaller,
-			       GTK_TYPE_NONE,
+		g_signal_new("look",
+				G_OBJECT_CLASS_TYPE (object_class),
+			       G_SIGNAL_RUN_FIRST,
+			       G_STRUCT_OFFSET(GtkMineFieldClass, look),
+			       NULL, NULL,
+			       g_cclosure_marshal_VOID__VOID,
+			       G_TYPE_NONE,
 			       0);
 	minefield_signals[UNLOOK_SIGNAL] =
-		gtk_signal_new("unlook",
-			       GTK_RUN_FIRST,
-			       GTK_CLASS_TYPE (object_class),
-			       GTK_SIGNAL_OFFSET(GtkMineFieldClass, unlook),
-			       gtk_signal_default_marshaller,
-			       GTK_TYPE_NONE,
+		g_signal_new("unlook",
+				G_OBJECT_CLASS_TYPE (object_class),
+			       G_SIGNAL_RUN_FIRST,
+			       G_STRUCT_OFFSET(GtkMineFieldClass, unlook),
+			       NULL, NULL,
+			       g_cclosure_marshal_VOID__VOID,
+			       G_TYPE_NONE,
 			       0);
 	minefield_signals[WIN_SIGNAL] =
-		gtk_signal_new("win",
-			       GTK_RUN_FIRST,
-			       GTK_CLASS_TYPE (object_class),
-			       GTK_SIGNAL_OFFSET(GtkMineFieldClass, win),
-			       gtk_signal_default_marshaller,
-			       GTK_TYPE_NONE,
+		g_signal_new("win",
+				G_OBJECT_CLASS_TYPE (object_class),
+			       G_SIGNAL_RUN_FIRST,
+			       G_STRUCT_OFFSET(GtkMineFieldClass, win),
+			       NULL, NULL,
+			       g_cclosure_marshal_VOID__VOID,
+			       G_TYPE_NONE,
 			       0);
-#if 0
-	gtk_object_class_add_signals(object_class, minefield_signals, LAST_SIGNAL);
-#endif
 }
 
 static void gtk_minefield_init (GtkMineField *mfield)
 {
-#ifndef GTK_HAVE_FEATURES_1_1_4
-  #if 0
-        GTK_WIDGET_SET_FLAGS (mfield, GTK_BASIC);
-  #endif
-#endif
         mfield->xsize = 0;
         mfield->ysize = 0;
 
         GTK_WIDGET (mfield)->requisition.width = mfield->minesize;
         GTK_WIDGET (mfield)->requisition.height = mfield->minesize;
+
+	gtk_minefield_setup_signs(GTK_WIDGET(mfield));
+
+	gtk_minefield_setup_numbers(mfield);
+}
+
+static gulong random_seed;
+
+static void init_random(gulong seed)
+{
+	random_seed = seed;
+}
+
+static gulong get_random(gulong limit)
+{
+	do {
+		random_seed = (random_seed*1139113+10921)>>2;
+	} while (random_seed > ((gulong)(G_MAXLONG/limit))*limit);
+	return random_seed % limit;
 }
 
 void gtk_minefield_set_size(GtkMineField *mfield, guint xsize, guint ysize)
@@ -780,42 +968,52 @@ void gtk_minefield_set_size(GtkMineField *mfield, guint xsize, guint ysize)
 		mfield->mines = g_realloc(mfield->mines,
 					  sizeof(mine)*xsize*ysize);
 	}
-	mfield->xsize = xsize;
-	mfield->ysize = ysize;
+
+	if (mfield->xsize != xsize || mfield->ysize != ysize) {
+		mfield->xsize = xsize;
+		mfield->ysize = ysize;
+		if (GTK_WIDGET_VISIBLE(mfield)) {
+			gtk_widget_queue_resize(GTK_WIDGET(mfield));
+		}
+	}
 }
 
 GtkWidget* gtk_minefield_new(void)
 {
         GtkMineField *mfield;
         
-	mfield = gtk_type_new(gtk_minefield_get_type());
+	mfield = GTK_MINEFIELD (g_object_new (TYPE_GTK_MINEFIELD,
+				NULL));
 	mfield->mines = NULL;
+	mfield->started = FALSE;
 	gtk_minefield_set_size(mfield, 0, 0);
 
 	mfield->cdown = -1;
-	mfield->numstr[0].gc = 0; /* Force GC generation */
 	return GTK_WIDGET(mfield);
 }
 
 guint gtk_minefield_get_type ()
 {
-        static guint minefield_type = 0;
+        static GType minefield_type = 0;
         
-        if (!minefield_type)
-        {
-                GtkTypeInfo minefield_info =
+	if (minefield_type == 0) {
+                static GTypeInfo minefield_info =
                         {
-                                "GtkMineField",
-                                sizeof (GtkMineField),
                                 sizeof (GtkMineFieldClass),
-                                (GtkClassInitFunc) gtk_minefield_class_init,
-                                (GtkObjectInitFunc) gtk_minefield_init,
-                                /* reserved_1 */ NULL,
-        			/* reserved_2 */ NULL,
-        			(GtkClassInitFunc) NULL,
+				NULL, /* base_init */
+				NULL, /* base_finalize */
+                                (GClassInitFunc) gtk_minefield_class_init,
+				NULL, /* class_finalize */
+				NULL, /* class_data */
+				sizeof (GtkMineField),
+				0, /* n_preallocs */
+                                (GInstanceInitFunc) gtk_minefield_init,
                         };
                         
-                        minefield_type = gtk_type_unique (gtk_widget_get_type (), &minefield_info);
+                        minefield_type = g_type_register_static (GTK_TYPE_WIDGET,
+					"GtkMineField",
+					&minefield_info,
+					0);
         }
         
         return minefield_type;
@@ -828,41 +1026,24 @@ void gtk_minefield_set_mines(GtkMineField *mfield, guint mcount, guint minesize)
 	g_return_if_fail(minesize>0);
 	
         mfield->mcount = mcount;
-        mfield->minesize = minesize;
-        mfield->numstr[0].gc = 0;
 
-	if (GTK_WIDGET_VISIBLE(mfield)) {
-		gtk_widget_queue_resize(GTK_WIDGET(mfield));
+	if (mfield->minesize != minesize) {
+		mfield->minesize = minesize;
+		gtk_minefield_setup_numbers(mfield);
+		if (GTK_WIDGET_VISIBLE(mfield)) {
+			gtk_widget_queue_resize(GTK_WIDGET(mfield));
+		}
 	}
 }
 
-static gulong random_seed;
-
-void init_random(gulong seed)
-{
-        random_seed = seed;
-}
-
-gulong get_random(gulong limit)
-{
-	do {
-		random_seed = (random_seed*1139113+10921)>>2;
-	} while (random_seed > ((gulong)(G_MAXLONG/limit))*limit);
-	return random_seed % limit;
-}
-
 void gtk_minefield_restart(GtkMineField *mfield)
 {
-	guint i, j;
-        guint x, y;
-	guint tmp;
-	guint n;
-	guint cidx;
+	guint i;
 
         g_return_if_fail(mfield != NULL);
         g_return_if_fail(GTK_IS_MINEFIELD(mfield));
 
-	mfield->flags = 0;
+	mfield->flag_count = 0;
 	mfield->shown = 0;
 	mfield->lose  = 0;
 	mfield->win   = 0;
@@ -871,41 +1052,23 @@ void gtk_minefield_restart(GtkMineField *mfield)
 	mfield->bdown[2] = 0;
         mfield->cdown = -1;
         mfield->multi_mode = 0;
+	mfield->in_play = 0;
 
-	for (i=0; i<mfield->mcount; i++) {
-		mfield->mines[i].mined = 1;
-	}
-	for (i=mfield->mcount; i<mfield->xsize*mfield->ysize; i++) {
-		mfield->mines[i].mined = 0;
+	/* set up mfield->mcount mines.  (randomize later) */
+	for (i=0; i<mfield->xsize * mfield->ysize; i++) {
+		mfield->mines[i].mined = i<mfield->mcount ? 1 : 0;
+		mfield->mines[i].marked = MINE_NOMARK;
+                mfield->mines[i].shown  = 0;
+		mfield->mines[i].down   = 0;
 	}
 
-	if (secs == 0) {
+	if (secs == 0) {	/* random not seeded yet */
 		time((time_t *)&secs);
 		init_random(secs);
 	}
-			
-	
-	for (i=0; i<mfield->xsize*mfield->ysize; i++) {
-                mfield->mines[i].marked = 0;
-                mfield->mines[i].shown  = 0;
-		mfield->mines[i].down   = 0;
-		j = (guint)get_random(mfield->xsize*mfield->ysize);
-		tmp = mfield->mines[i].mined;
-		mfield->mines[i].mined = mfield->mines[j].mined;
-		mfield->mines[j].mined = tmp;
-	}
-
-	for (x=0; x<mfield->xsize; x++) {
-		for (y=0; y<mfield->ysize; y++) {
-			n = 0;
-			for (i=0; i<8; i++) {
-				if ((cidx = cell_idx(mfield, x+neighbour_map[i].x,
-						     y+neighbour_map[i].y)) != -1) {
-					if (mfield->mines[cidx].mined) n++;
-				}
-			}
-                        mfield->mines[x+mfield->xsize*y].neighbours = n;
-		}
-	}
+	if (mfield->started == FALSE)
+		mfield->started = TRUE;
+	else
+		gtk_minefield_draw(mfield, NULL);
 }
 
