@@ -241,25 +241,23 @@ void quit_game_maybe(GtkWidget *widget, gint button)
 	}
 }
 
-void quit_game_cb(GtkWidget *widget, gpointer data)
+gboolean quit_game_cb(GtkWidget *widget, gpointer data)
 {
 	GtkWidget *dialog;
 
 	if(game_in_progress) {
 		gint response;
 
-		dialog = gtk_dialog_new_with_buttons (NULL,
-				GTK_WINDOW (window),
+		dialog = gtk_message_dialog_new (GTK_WINDOW (window),
 				GTK_DIALOG_MODAL,
-				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
-				GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-				NULL);
-		{
-			GtkWidget *label = gtk_label_new (_("Do you really want to quit?"));
-			gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
-					label);
-		}
+				GTK_MESSAGE_QUESTION,
+				GTK_BUTTONS_NONE,
+				_("Really quit GNOME Iagno?"), NULL);
 
+		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+					GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+					GTK_STOCK_QUIT, GTK_RESPONSE_ACCEPT, 
+					NULL);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 				GTK_RESPONSE_REJECT);
 
@@ -268,9 +266,15 @@ void quit_game_cb(GtkWidget *widget, gpointer data)
 		gtk_widget_destroy (dialog);
 
 		if (response == GTK_RESPONSE_ACCEPT)
+		{
 			quit_game_maybe(NULL, 0);
+			return FALSE;
+		} else {
+			return TRUE;
+		}
 	} else {
-		quit_game_maybe(NULL, 0);
+	  quit_game_maybe(NULL, 0);
+	  return FALSE;
 	}
 }
 
@@ -414,7 +418,7 @@ void about_cb(GtkWidget *widget, gpointer data)
 		char *filename = NULL;
 
 		filename = gnome_program_locate_file (NULL,
-				GNOME_FILE_DOMAIN_PIXMAP,  ("iagno.png"),
+				GNOME_FILE_DOMAIN_APP_PIXMAP,  ("iagno.png"),
 				TRUE, NULL);
 		if (filename != NULL)
 		{
@@ -524,7 +528,7 @@ void load_pixmaps()
 	g_return_if_fail (tile_set != NULL && tile_set [0] != '0');
 
 	tmp = g_strconcat("iagno/", tile_set, NULL);
-	fname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+	fname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
 			tmp, FALSE, NULL);
 	g_free(tmp);
 
@@ -713,7 +717,7 @@ void create_window()
 
 	gtk_widget_realize(window);
 	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
-	g_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(quit_game_cb), NULL);
+	g_signal_connect(GTK_OBJECT(window), "delete_event", G_CALLBACK(quit_game_cb), NULL);
 
 	gnome_app_create_menus(GNOME_APP(window), mainmenu);
 
@@ -912,7 +916,7 @@ int main(int argc, char **argv)
 	gtk_object_sink(GTK_OBJECT(client));
 
 	g_signal_connect(GTK_OBJECT(client), "save_yourself", GTK_SIGNAL_FUNC(save_state), argv[0]);
-	g_signal_connect(GTK_OBJECT(client), "die", GTK_SIGNAL_FUNC(quit_game_cb), argv[0]);
+	g_signal_connect(GTK_OBJECT(client), "die", G_CALLBACK(quit_game_cb), argv[0]);
 
 	create_window();
 	
